/////////////////////////////////////////////////////////////////////////////
//  Copyright (C) 2002 Ultr@VNC. All Rights Reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
// If the source code for the program is not available from the place from
// which you received this file, check 
// http://ultravnc.sourceforge.net/
//
////////////////////////////////////////////////////////////////////////////
//
// IF YOU USE THIS GPL SOURCE CODE TO MAKE YOUR DSM PLUGIN, PLEASE ADD YOUR
// COPYRIGHT TO THE TOP OF THIS FILE AND SHORTLY DESCRIBE/EXPLAIN THE 
// MODIFICATIONS YOU'VE MADE. THANKS.
//
// IF YOU DON'T USE THIS CODE AS A BASE FOR YOUR PLUGIN, THE HEADER ABOVE AND
// ULTR@VNC COPYRIGHT SHOULDN'T BE FOUND IN YOUR PLUGIN SOURCE CODE.
//
////////////////////////////////////////////////////////////////////////////
//
// UltraRC4Plugin - RC4ÍvOC
// 
// Copyright(C) K_Yaguchi
// All rights reserved.
// 
//////////////////////////////////////////////////////////////////////////////

#ifndef _H_ULTRARC4PLUGIN_
#define _H_ULTRARC4PLUGIN_

#define _CRT_SECURE_NO_WARNINGS

#include <windows.h>
#include <shlwapi.h>
#include <stdio.h>
#include "DlgConfig.h"
#include "librc4.h"
#include "resource.h"

#pragma comment(lib,"shlwapi.lib")



///////////////////////////////////////////////////////
//
#define URC4KEY     "urc4.key"    // ʌt@C
#define URC4KEYLEN  256           // t@C


///////////////////////////////////////////////////////
//

// obt@\
typedef struct
{
	BYTE* pBuffer;  // obt@
	int   nSize;    // obt@TCY
} PLGBUFFER, *LPPLGBUFFER;


// XbhpbNNX
class CThreadLock
{
public:
	CThreadLock() { InitializeCriticalSection(&m_LockObj); };
	virtual ~CThreadLock() { DeleteCriticalSection(&m_LockObj); };
	void Enter() { EnterCriticalSection(&m_LockObj); };
	void Leave() { LeaveCriticalSection(&m_LockObj); };
private:
	CRITICAL_SECTION m_LockObj;
};


// vZXpbNNX
#define WRITELOCK "UltraRC4Plugin@Mutex"
class CProcessLock
{
public:
	CProcessLock() { m_hMutex=NULL; };
	virtual ~CProcessLock() { Leave(); };
	void Enter()
	{
		m_hMutex = OpenMutex(MUTEX_ALL_ACCESS, FALSE, WRITELOCK);
		if( m_hMutex )
			WaitForSingleObject(m_hMutex, INFINITE);
		else
			m_hMutex = CreateMutex(NULL, TRUE, WRITELOCK);
	};
	void Leave()
	{
		if( m_hMutex )
			ReleaseMutex( m_hMutex );
		m_hMutex=NULL;
	};
private:
	HANDLE m_hMutex;
};


///////////////////////////////////////////////////////
//

// Plugin interface
class IPlugin
{
public:
	virtual ~IPlugin() {};

	virtual BYTE* TransformBuffer(BYTE* pDataBuffer, int nDataLen, int* pnTransformedDataLen) = 0;
	virtual BYTE* RestoreBuffer(BYTE* pRestoredDataBuffer, int nDataLen, int* pnRestoredDataLen) = 0;
};

// UltraRC4PluginpNX
class IUltraRC4Plugin : public IPlugin
{
public:
	IUltraRC4Plugin(const BYTE* pKeyData);
	virtual ~IUltraRC4Plugin();

	BYTE* TransformBuffer(BYTE* pDataBuffer, int nDataLen, int* pnTransformedDataLen);
	BYTE* RestoreBuffer(BYTE* pRestoredDataBuffer, int nDataLen, int* pnRestoredDataLen);

private:
	PLGBUFFER m_TransBuffer;     // Transformpobt@
	PLGBUFFER m_RestBuffer;      // Restorepobt@
	RC4STATE  m_rc4Trans;        // 
	RC4STATE  m_rc4Rest;         // 
};




///////////////////////////////////////////////////////
//
int PluginMallocBuffer(PLGBUFFER* pPlgBuffer, int nNewSize);
void outlog(const char* str, ...);
void outbin(const BYTE* pData, int nSize, int nType);

void BeginHook(HINSTANCE hInstance);
void EndHook(void);


///////////////////////////////////////////////////////
//
#if 0
#define LOG outlog
#else
#define LOG __noop
#endif
#if 0
#define BIN outbin
#else
#define BIN __noop
#endif

///////////////////////////////////////////////////////
//

#ifdef ULTRARC4PLUGIN_EXPORTS
#define PLUGIN_API __declspec(dllexport)
#else
#define PLUGIN_API __declspec(dllimport)
#endif

//
// A DSM Plugin MUST export (extern "C" - __cdecl) all the following functions
// (same names, same signatures)

extern "C"
{
	PLUGIN_API char* Description(void);
	PLUGIN_API int Startup(void);
	PLUGIN_API int Shutdown(void);
	PLUGIN_API int Reset(void);
	PLUGIN_API int SetParams(HWND hVNC, char* szParams);
	PLUGIN_API char* GetParams(void);
	PLUGIN_API BYTE* TransformBuffer(BYTE* pDataBuffer, int nDataLen, int* pnTransformedDataLen);
	PLUGIN_API BYTE* RestoreBuffer(BYTE* pTransBuffer, int nTransDataLen, int* pnRestoredDataLen);
	PLUGIN_API void FreeBuffer(BYTE* pBuffer);
	PLUGIN_API IPlugin* CreatePluginInterface(void);
}

#endif // _H_ULTRARC4PLUGIN_
